@@ -21,6 +21,7 @@ from cliff.command import Command
 from cliff.hooks import CommandHook
 
 from kayobe import ansible
+from kayobe import environment
 from kayobe import kolla_ansible
 from kayobe import utils
 from kayobe import vault
@@ -413,7 +414,7 @@ class SeedHypervisorHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
     * Configure user accounts, group associations, and authorised SSH keys.
     * Configure the host's network interfaces.
     * Set sysctl parameters.
-    * Configure timezone.
+    * Configure timezone and ntp.
     * Optionally, configure software RAID arrays.
     * Optionally, configure encryption.
     * Configure LVM volumes.
@@ -431,6 +432,16 @@ class SeedHypervisorHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
 
     def take_action(self, parsed_args):
         self.app.LOG.debug("Configuring seed hypervisor host OS")
+        # Explicitly request the dump-config tag to ensure this play runs even
+        # if the user specified tags.
+        ansible_user = self.run_kayobe_config_dump(
+            parsed_args, host="seed-hypervisor",
+            var_name="kayobe_ansible_user", tags="dump-config")
+        if not ansible_user:
+            self.app.LOG.error("Could not determine kayobe_ansible_user "
+                               "variable for seed hypervisor host")
+            sys.exit(1)
+
         # Allocate IP addresses.
         playbooks = _build_playbook_list("ip-allocation")
         self.run_kayobe_playbooks(parsed_args, playbooks,
@@ -442,7 +453,7 @@ class SeedHypervisorHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
         if parsed_args.wipe_disks:
             playbooks += _build_playbook_list("wipe-disks")
         playbooks += _build_playbook_list(
-            "users", "dev-tools", "network", "sysctl", "timezone",
+            "users", "dev-tools", "network", "sysctl", "time",
             "mdadm", "luks", "lvm", "seed-hypervisor-libvirt-host")
         self.run_kayobe_playbooks(parsed_args, playbooks,
                                   limit="seed-hypervisor")
@@ -484,12 +495,15 @@ class SeedHypervisorHostCommandRun(KayobeAnsibleMixin, VaultMixin, Command):
         group = parser.add_argument_group("Host Command Run")
         group.add_argument("--command", required=True,
                            help="Command to run (required).")
+        group.add_argument("--show-output", action='store_true',
+                           help="Show command output")
         return parser
 
     def take_action(self, parsed_args):
         self.app.LOG.debug("Run command on seed hypervisor host")
         extra_vars = {
-            "host_command_to_run": utils.escape_jinja(parsed_args.command)}
+            "host_command_to_run": utils.escape_jinja(parsed_args.command),
+            "show_output": parsed_args.show_output}
         playbooks = _build_playbook_list("host-command-run")
         self.run_kayobe_playbooks(parsed_args, playbooks,
                                   limit="seed-hypervisor",
@@ -560,7 +574,7 @@ class SeedHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
     * Set sysctl parameters.
     * Configure IP routing and source NAT.
     * Disable bootstrap interface configuration.
-    * Configure timezone.
+    * Configure timezone and ntp.
     * Optionally, configure software RAID arrays.
     * Optionally, configure encryption.
     * Configure LVM volumes.
@@ -594,7 +608,7 @@ class SeedHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
             playbooks += _build_playbook_list("wipe-disks")
         playbooks += _build_playbook_list(
             "users", "dev-tools", "disable-selinux", "network",
-            "sysctl", "ip-routing", "snat", "disable-glean", "timezone",
+            "sysctl", "ip-routing", "snat", "disable-glean", "time",
             "mdadm", "luks", "lvm", "docker-devicemapper",
             "kolla-ansible-user", "kolla-pip", "kolla-target-venv")
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="seed")
@@ -649,12 +663,15 @@ class SeedHostCommandRun(KayobeAnsibleMixin, VaultMixin, Command):
         group = parser.add_argument_group("Host Command Run")
         group.add_argument("--command", required=True,
                            help="Command to run (required).")
+        group.add_argument("--show-output", action='store_true',
+                           help="Show command output")
         return parser
 
     def take_action(self, parsed_args):
         self.app.LOG.debug("Run command on seed host")
         extra_vars = {
-            "host_command_to_run": utils.escape_jinja(parsed_args.command)}
+            "host_command_to_run": utils.escape_jinja(parsed_args.command),
+            "show_output": parsed_args.show_output}
         playbooks = _build_playbook_list("host-command-run")
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="seed",
                                   extra_vars=extra_vars)
@@ -750,6 +767,8 @@ class SeedContainerImageBuild(KayobeAnsibleMixin, VaultMixin, Command):
         parser = super(SeedContainerImageBuild, self).get_parser(
             prog_name)
         group = parser.add_argument_group("Container Image Build")
+        group.add_argument("--nocache", action="store_true",
+                           help="whether to not use cache")
         group.add_argument("--push", action="store_true",
                            help="whether to push images to a registry after "
                                 "building")
@@ -763,7 +782,10 @@ class SeedContainerImageBuild(KayobeAnsibleMixin, VaultMixin, Command):
         playbooks = _build_playbook_list(
             "container-image-builders-check", "kolla-build",
             "container-image-build")
-        extra_vars = {"push_images": parsed_args.push}
+        extra_vars = {
+            "nocache": parsed_args.nocache,
+            "push_images": parsed_args.push
+        }
         if parsed_args.regex:
             regexes = " ".join(parsed_args.regex)
             extra_vars["container_image_regexes"] = regexes
@@ -910,22 +932,6 @@ class OvercloudDeprovision(KayobeAnsibleMixin, VaultMixin, Command):
         self.run_kayobe_playbooks(parsed_args, playbooks)
 
 
-class OvercloudFactsGather(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
-                           Command):
-    """Gather facts for Kayobe and Kolla Ansible."""
-
-    def take_action(self, parsed_args):
-        self.app.LOG.debug("Gathering overcloud host facts")
-
-        # Gather facts for Kayobe.
-        playbooks = _build_playbook_list("overcloud-facts-gather")
-        self.run_kayobe_playbooks(parsed_args, playbooks)
-
-        # Gather facts for Kolla Ansible.
-        self.generate_kolla_ansible_config(parsed_args, service_config=False)
-        self.run_kolla_ansible_overcloud(parsed_args, "gather-facts")
-
-
 class OvercloudHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
                              Command):
     """Configure the overcloud host OS and services.
@@ -942,7 +948,7 @@ class OvercloudHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
     * Configure the host's network interfaces.
     * Set sysctl parameters.
     * Disable bootstrap interface configuration.
-    * Configure timezone.
+    * Configure timezone and ntp.
     * Optionally, configure software RAID arrays.
     * Optionally, configure encryption.
     * Configure LVM volumes.
@@ -975,7 +981,7 @@ class OvercloudHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
             playbooks += _build_playbook_list("wipe-disks")
         playbooks += _build_playbook_list(
             "users", "dev-tools", "disable-selinux", "network",
-            "sysctl", "disable-glean", "disable-cloud-init", "timezone",
+            "sysctl", "disable-glean", "disable-cloud-init", "time",
             "mdadm", "luks", "lvm", "docker-devicemapper",
             "kolla-ansible-user", "kolla-pip", "kolla-target-venv")
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud")
@@ -1024,18 +1030,22 @@ class OvercloudHostCommandRun(KayobeAnsibleMixin, VaultMixin, Command):
         group = parser.add_argument_group("Host Command Run")
         group.add_argument("--command", required=True,
                            help="Command to run (required).")
+        group.add_argument("--show-output", action='store_true',
+                           help="Show command output")
         return parser
 
     def take_action(self, parsed_args):
         self.app.LOG.debug("Run command on overcloud host")
         extra_vars = {
-            "host_command_to_run": utils.escape_jinja(parsed_args.command)}
+            "host_command_to_run": utils.escape_jinja(parsed_args.command),
+            "show_output": parsed_args.show_output}
         playbooks = _build_playbook_list("host-command-run")
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud",
                                   extra_vars=extra_vars)
 
 
-class OvercloudHostUpgrade(KayobeAnsibleMixin, VaultMixin, Command):
+class OvercloudHostUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
+                           Command):
     """Upgrade the overcloud host services.
 
     Performs the changes necessary to make the host services suitable for the
@@ -1049,6 +1059,15 @@ class OvercloudHostUpgrade(KayobeAnsibleMixin, VaultMixin, Command):
             "overcloud-docker-sdk-upgrade", "overcloud-etc-hosts-fixup")
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud")
 
+        # TODO(mgoddard): Remove this in Y cycle after Kolla Ansible chrony
+        # container has been dropped for a cycle.
+        # NOTE(mgoddard): Clean up the chrony container if it exists, and
+        # deploy a host chrony daemon.
+        self.generate_kolla_ansible_config(parsed_args, service_config=False)
+        self.run_kolla_ansible_overcloud(parsed_args, "chrony-cleanup")
+        playbooks = _build_playbook_list("time")
+        self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud")
+
 
 class OvercloudDatabaseBackup(KollaAnsibleMixin, KayobeAnsibleMixin,
                               VaultMixin, Command):
@@ -1064,10 +1083,6 @@ class OvercloudDatabaseBackup(KollaAnsibleMixin, KayobeAnsibleMixin,
 
     def take_action(self, parsed_args):
         self.app.LOG.debug("Performing overcloud database backup")
-
-        # First prepare configuration.
-        self.generate_kolla_ansible_config(parsed_args, service_config=False)
-
         extra_args = []
         if parsed_args.incremental:
             extra_args.append('--incremental')
@@ -1092,10 +1107,6 @@ class OvercloudDatabaseRecover(KollaAnsibleMixin, KayobeAnsibleMixin,
     def take_action(self, parsed_args):
         self.app.LOG.debug("Performing overcloud database recovery")
         extra_vars = {}
-
-        # First prepare configuration.
-        self.generate_kolla_ansible_config(parsed_args, service_config=True)
-
         if parsed_args.force_recovery_host:
             extra_vars['mariadb_recover_inventory_name'] = (
                 parsed_args.force_recovery_host)
@@ -1282,6 +1293,29 @@ class OvercloudServiceDeployContainers(KollaAnsibleMixin, KayobeAnsibleMixin,
                                   extra_vars=extra_vars, limit="overcloud")
 
 
+class OvercloudServicePrechecks(KollaAnsibleMixin, KayobeAnsibleMixin,
+                                VaultMixin, Command):
+    """Run prechecks against overcloud services.
+
+    * Configure kolla-ansible.
+    * Configure overcloud services in kolla-ansible.
+    * Perform kolla-ansible prechecks to verify the system state for
+      deployment.
+
+    This can be used in conjunction with the --tags and --kolla-tags arguments
+    to check specific services.
+    """
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Running overcloud prechecks")
+
+        # First prepare configuration.
+        self.generate_kolla_ansible_config(parsed_args)
+
+        # Run the kolla-ansible prechecks.
+        self.run_kolla_ansible_overcloud(parsed_args, "prechecks")
+
+
 class OvercloudServiceReconfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
                                   VaultMixin, Command):
     """Reconfigure the overcloud services.
@@ -1497,6 +1531,8 @@ class OvercloudContainerImageBuild(KayobeAnsibleMixin, VaultMixin, Command):
         parser = super(OvercloudContainerImageBuild, self).get_parser(
             prog_name)
         group = parser.add_argument_group("Container Image Build")
+        group.add_argument("--nocache", action="store_true",
+                           help="whether to not use cache")
         group.add_argument("--push", action="store_true",
                            help="whether to push images to a registry after "
                                 "building")
@@ -1510,7 +1546,10 @@ class OvercloudContainerImageBuild(KayobeAnsibleMixin, VaultMixin, Command):
         playbooks = _build_playbook_list(
             "container-image-builders-check", "kolla-build",
             "container-image-build")
-        extra_vars = {"push_images": parsed_args.push}
+        extra_vars = {
+            "nocache": parsed_args.nocache,
+            "push_images": parsed_args.push
+        }
         if parsed_args.regex:
             regexes = " ".join(parsed_args.regex)
             extra_vars["container_image_regexes"] = regexes
@@ -1677,7 +1716,7 @@ class BaremetalComputeSerialConsoleDisable(BaremetalComputeSerialConsoleBase):
 
 class BaremetalComputeUpdateDeploymentImage(KayobeAnsibleMixin, VaultMixin,
                                             Command):
-    """Update the Ironic nodes to use the new  kernel and ramdisk images."""
+    """Update the Ironic nodes to use the new kernel and ramdisk images."""
 
     def get_parser(self, prog_name):
         parser = super(BaremetalComputeUpdateDeploymentImage, self).get_parser(
@@ -1691,7 +1730,7 @@ class BaremetalComputeUpdateDeploymentImage(KayobeAnsibleMixin, VaultMixin,
 
     def take_action(self, parsed_args):
         self.app.LOG.debug(
-            "Upgrading the ironic nodes to use the latest  deployment images")
+            "Upgrading the ironic nodes to use the latest deployment images")
         playbooks = _build_playbook_list("overcloud-ipa-images")
         extra_vars = {}
         extra_vars["ipa_images_update_ironic_nodes"] = True
@@ -1701,3 +1740,34 @@ class BaremetalComputeUpdateDeploymentImage(KayobeAnsibleMixin, VaultMixin,
             )
         self.run_kayobe_playbooks(parsed_args, playbooks,
                                   extra_vars=extra_vars)
+
+
+class EnvironmentCreate(KayobeAnsibleMixin, VaultMixin, Command):
+    """Create a new Kayobe environment."""
+
+    def get_parser(self, prog_name):
+        parser = super(EnvironmentCreate, self).get_parser(prog_name)
+        group = parser.add_argument_group("Kayobe Environments")
+        environment.add_args(group)
+        return parser
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Creating new Kayobe environment")
+        if not parsed_args.environment:
+            self.app.LOG.error("An environment must be specified")
+            sys.exit(1)
+
+        source_config_path = parsed_args.source_config_path
+        if source_config_path:
+            result = utils.is_readable_dir(source_config_path)
+            if not result["result"]:
+                self.app.LOG.error("Kayobe configuration %s is invalid: %s",
+                                   source_config_path, result["message"])
+                sys.exit(1)
+
+        try:
+            environment.create_kayobe_environment(parsed_args)
+        except Exception as e:
+            self.app.LOG.error("Failed to create environment %s: %s",
+                               parsed_args.environment, repr(e))
+            sys.exit(1)
