@@ -286,16 +286,25 @@ class TestCase(unittest.TestCase):
         ]
         self.assertEqual(expected_calls, mock_run.call_args_list)
 
+    @mock.patch.object(commands.KayobeAnsibleMixin,
+                       "run_kayobe_config_dump")
     @mock.patch.object(commands.KayobeAnsibleMixin,
                        "run_kayobe_playbooks")
-    def test_seed_hypervisor_host_configure(self, mock_run):
+    def test_seed_hypervisor_host_configure(self, mock_run, mock_dump):
         command = commands.SeedHypervisorHostConfigure(TestApp(), [])
         parser = command.get_parser("test")
         parsed_args = parser.parse_args([])
+        mock_dump.return_value = "stack"
 
         result = command.run(parsed_args)
         self.assertEqual(0, result)
 
+        expected_calls = [
+            mock.call(mock.ANY, host="seed-hypervisor",
+                      var_name="kayobe_ansible_user", tags="dump-config")
+        ]
+        self.assertEqual(expected_calls, mock_dump.call_args_list)
+
         expected_calls = [
             mock.call(
                 mock.ANY,
@@ -316,7 +325,7 @@ class TestCase(unittest.TestCase):
                     utils.get_data_files_path("ansible", "dev-tools.yml"),
                     utils.get_data_files_path("ansible", "network.yml"),
                     utils.get_data_files_path("ansible", "sysctl.yml"),
-                    utils.get_data_files_path("ansible", "timezone.yml"),
+                    utils.get_data_files_path("ansible", "time.yml"),
                     utils.get_data_files_path("ansible", "mdadm.yml"),
                     utils.get_data_files_path("ansible", "luks.yml"),
                     utils.get_data_files_path("ansible", "lvm.yml"),
@@ -333,7 +342,8 @@ class TestCase(unittest.TestCase):
     def test_seed_hypervisor_host_command_run(self, mock_run):
         command = commands.SeedHypervisorHostCommandRun(TestApp(), [])
         parser = command.get_parser("test")
-        parsed_args = parser.parse_args(["--command", "ls -a"])
+        parsed_args = parser.parse_args(["--command", "ls -a",
+                                         "--show-output"])
 
         result = command.run(parsed_args)
         self.assertEqual(0, result)
@@ -347,7 +357,8 @@ class TestCase(unittest.TestCase):
                 ],
                 limit="seed-hypervisor",
                 extra_vars={
-                    "host_command_to_run": utils.escape_jinja("ls -a")},
+                    "host_command_to_run": utils.escape_jinja("ls -a"),
+                    "show_output": True}
             ),
         ]
         self.assertEqual(expected_calls, mock_run.call_args_list)
@@ -489,7 +500,7 @@ class TestCase(unittest.TestCase):
                     utils.get_data_files_path("ansible", "ip-routing.yml"),
                     utils.get_data_files_path("ansible", "snat.yml"),
                     utils.get_data_files_path("ansible", "disable-glean.yml"),
-                    utils.get_data_files_path("ansible", "timezone.yml"),
+                    utils.get_data_files_path("ansible", "time.yml"),
                     utils.get_data_files_path("ansible", "mdadm.yml"),
                     utils.get_data_files_path("ansible", "luks.yml"),
                     utils.get_data_files_path("ansible", "lvm.yml"),
@@ -542,7 +553,8 @@ class TestCase(unittest.TestCase):
     def test_seed_host_command_run(self, mock_run):
         command = commands.SeedHostCommandRun(TestApp(), [])
         parser = command.get_parser("test")
-        parsed_args = parser.parse_args(["--command", "ls -a"])
+        parsed_args = parser.parse_args(["--command", "ls -a",
+                                         "--show-output"])
 
         result = command.run(parsed_args)
         self.assertEqual(0, result)
@@ -556,7 +568,8 @@ class TestCase(unittest.TestCase):
                 ],
                 limit="seed",
                 extra_vars={
-                    "host_command_to_run": utils.escape_jinja("ls -a")},
+                    "host_command_to_run": utils.escape_jinja("ls -a"),
+                    "show_output": True}
             ),
         ]
         self.assertEqual(expected_calls, mock_run.call_args_list)
@@ -685,6 +698,7 @@ class TestCase(unittest.TestCase):
                     "container_image_sets": (
                         "{{ seed_container_image_sets }}"),
                     "push_images": False,
+                    "nocache": False
                 }
             ),
         ]
@@ -711,6 +725,35 @@ class TestCase(unittest.TestCase):
                 extra_vars={
                     "container_image_regexes": "^regex1$ ^regex2$",
                     "push_images": True,
+                    "nocache": False
+                }
+            ),
+        ]
+        self.assertEqual(expected_calls, mock_run.call_args_list)
+
+    @mock.patch.object(commands.KayobeAnsibleMixin,
+                       "run_kayobe_playbooks")
+    def test_seed_container_image_build_with_nocache(self, mock_run):
+        command = commands.SeedContainerImageBuild(TestApp(), [])
+        parser = command.get_parser("test")
+        parsed_args = parser.parse_args(["--nocache"])
+        result = command.run(parsed_args)
+        self.assertEqual(0, result)
+        expected_calls = [
+            mock.call(
+                mock.ANY,
+                [
+                    utils.get_data_files_path(
+                        "ansible", "container-image-builders-check.yml"),
+                    utils.get_data_files_path("ansible", "kolla-build.yml"),
+                    utils.get_data_files_path(
+                        "ansible", "container-image-build.yml")
+                ],
+                extra_vars={
+                    "container_image_sets": (
+                        "{{ seed_container_image_sets }}"),
+                    "push_images": False,
+                    "nocache": True
                 }
             ),
         ]
@@ -965,43 +1008,6 @@ class TestCase(unittest.TestCase):
         ]
         self.assertEqual(expected_calls, mock_run.call_args_list)
 
-    @mock.patch.object(commands.KayobeAnsibleMixin,
-                       "run_kayobe_playbooks")
-    @mock.patch.object(commands.KollaAnsibleMixin,
-                       "run_kolla_ansible_overcloud")
-    def test_overcloud_facts_gather(self, mock_kolla_run, mock_run):
-        command = commands.OvercloudFactsGather(TestApp(), [])
-        parser = command.get_parser("test")
-        parsed_args = parser.parse_args([])
-
-        result = command.run(parsed_args)
-        self.assertEqual(0, result)
-
-        expected_calls = [
-            mock.call(
-                mock.ANY,
-                [
-                    utils.get_data_files_path(
-                        "ansible", "overcloud-facts-gather.yml"),
-                ],
-            ),
-            mock.call(
-                mock.ANY,
-                [utils.get_data_files_path("ansible", "kolla-ansible.yml")],
-                tags="config",
-                ignore_limit=True,
-            ),
-        ]
-        self.assertEqual(expected_calls, mock_run.call_args_list)
-
-        expected_calls = [
-            mock.call(
-                mock.ANY,
-                "gather-facts"
-            ),
-        ]
-        self.assertEqual(expected_calls, mock_kolla_run.call_args_list)
-
     @mock.patch.object(commands.KayobeAnsibleMixin,
                        "run_kayobe_playbooks")
     @mock.patch.object(commands.KollaAnsibleMixin,
@@ -1039,7 +1045,7 @@ class TestCase(unittest.TestCase):
                     utils.get_data_files_path("ansible", "disable-glean.yml"),
                     utils.get_data_files_path(
                         "ansible", "disable-cloud-init.yml"),
-                    utils.get_data_files_path("ansible", "timezone.yml"),
+                    utils.get_data_files_path("ansible", "time.yml"),
                     utils.get_data_files_path("ansible", "mdadm.yml"),
                     utils.get_data_files_path("ansible", "luks.yml"),
                     utils.get_data_files_path("ansible", "lvm.yml"),
@@ -1085,7 +1091,8 @@ class TestCase(unittest.TestCase):
     def test_overcloud_host_command_run(self, mock_run):
         command = commands.OvercloudHostCommandRun(TestApp(), [])
         parser = command.get_parser("test")
-        parsed_args = parser.parse_args(["--command", "ls -a"])
+        parsed_args = parser.parse_args(["--command", "ls -a",
+                                         "--show-output"])
 
         result = command.run(parsed_args)
         self.assertEqual(0, result)
@@ -1099,7 +1106,8 @@ class TestCase(unittest.TestCase):
                 ],
                 limit="overcloud",
                 extra_vars={
-                    "host_command_to_run": utils.escape_jinja("ls -a")},
+                    "host_command_to_run": utils.escape_jinja("ls -a"),
+                    "show_output": True}
             ),
         ]
         self.assertEqual(expected_calls, mock_run.call_args_list)
@@ -1184,7 +1192,9 @@ class TestCase(unittest.TestCase):
 
     @mock.patch.object(commands.KayobeAnsibleMixin,
                        "run_kayobe_playbooks")
-    def test_overcloud_host_upgrade(self, mock_run):
+    @mock.patch.object(commands.KollaAnsibleMixin,
+                       "run_kolla_ansible_overcloud")
+    def test_overcloud_host_upgrade(self, mock_kolla_run, mock_run):
         command = commands.OvercloudHostUpgrade(TestApp(), [])
         parser = command.get_parser("test")
         parsed_args = parser.parse_args([])
@@ -1207,30 +1217,39 @@ class TestCase(unittest.TestCase):
                 ],
                 limit="overcloud",
             ),
+            mock.call(
+                mock.ANY,
+                [utils.get_data_files_path("ansible", "kolla-ansible.yml")],
+                ignore_limit=True,
+                tags="config",
+            ),
+            mock.call(
+                mock.ANY,
+                [
+                    utils.get_data_files_path(
+                        "ansible", "time.yml"),
+                ],
+                limit="overcloud",
+            ),
         ]
         self.assertEqual(expected_calls, mock_run.call_args_list)
 
-    @mock.patch.object(commands.KayobeAnsibleMixin,
-                       "run_kayobe_playbooks")
+        expected_calls = [
+            mock.call(
+                mock.ANY,
+                "chrony-cleanup",
+            ),
+        ]
+        self.assertEqual(expected_calls, mock_kolla_run.call_args_list)
+
     @mock.patch.object(commands.KollaAnsibleMixin,
                        "run_kolla_ansible_overcloud")
-    def test_overcloud_database_backup(self, mock_kolla_run, mock_run):
+    def test_overcloud_database_backup(self, mock_run):
         command = commands.OvercloudDatabaseBackup(TestApp(), [])
         parser = command.get_parser("test")
         parsed_args = parser.parse_args([])
         result = command.run(parsed_args)
         self.assertEqual(0, result)
-
-        expected_calls = [
-            mock.call(
-                mock.ANY,
-                [utils.get_data_files_path("ansible", "kolla-ansible.yml")],
-                tags="config",
-                ignore_limit=True,
-            ),
-        ]
-        self.assertEqual(expected_calls, mock_run.call_args_list)
-
         expected_calls = [
             mock.call(
                 mock.ANY,
@@ -1238,30 +1257,16 @@ class TestCase(unittest.TestCase):
                 extra_args=[]
             ),
         ]
-        self.assertEqual(expected_calls, mock_kolla_run.call_args_list)
+        self.assertEqual(expected_calls, mock_run.call_args_list)
 
-    @mock.patch.object(commands.KayobeAnsibleMixin,
-                       "run_kayobe_playbooks")
     @mock.patch.object(commands.KollaAnsibleMixin,
                        "run_kolla_ansible_overcloud")
-    def test_overcloud_database_backup_incremental(self, mock_kolla_run,
-                                                   mock_run):
+    def test_overcloud_database_backup_incremental(self, mock_run):
         command = commands.OvercloudDatabaseBackup(TestApp(), [])
         parser = command.get_parser("test")
         parsed_args = parser.parse_args(["--incremental"])
         result = command.run(parsed_args)
         self.assertEqual(0, result)
-
-        expected_calls = [
-            mock.call(
-                mock.ANY,
-                [utils.get_data_files_path("ansible", "kolla-ansible.yml")],
-                tags="config",
-                ignore_limit=True,
-            ),
-        ]
-        self.assertEqual(expected_calls, mock_run.call_args_list)
-
         expected_calls = [
             mock.call(
                 mock.ANY,
@@ -1269,37 +1274,16 @@ class TestCase(unittest.TestCase):
                 extra_args=["--incremental"]
             ),
         ]
-        self.assertEqual(expected_calls, mock_kolla_run.call_args_list)
+        self.assertEqual(expected_calls, mock_run.call_args_list)
 
-    @mock.patch.object(commands.KayobeAnsibleMixin,
-                       "run_kayobe_playbooks")
     @mock.patch.object(commands.KollaAnsibleMixin,
                        "run_kolla_ansible_overcloud")
-    def test_overcloud_database_recover(self, mock_kolla_run, mock_run):
+    def test_overcloud_database_recover(self, mock_run):
         command = commands.OvercloudDatabaseRecover(TestApp(), [])
         parser = command.get_parser("test")
         parsed_args = parser.parse_args([])
         result = command.run(parsed_args)
         self.assertEqual(0, result)
-
-        expected_calls = [
-            mock.call(
-                mock.ANY,
-                [utils.get_data_files_path("ansible", "kolla-ansible.yml")],
-                tags="config",
-                ignore_limit=True,
-            ),
-            mock.call(
-                mock.ANY,
-                [
-                    utils.get_data_files_path("ansible",
-                                              "kolla-openstack.yml"),
-                ],
-                ignore_limit=True,
-            ),
-        ]
-        self.assertEqual(expected_calls, mock_run.call_args_list)
-
         expected_calls = [
             mock.call(
                 mock.ANY,
@@ -1307,38 +1291,16 @@ class TestCase(unittest.TestCase):
                 extra_vars={}
             ),
         ]
-        self.assertEqual(expected_calls, mock_kolla_run.call_args_list)
+        self.assertEqual(expected_calls, mock_run.call_args_list)
 
-    @mock.patch.object(commands.KayobeAnsibleMixin,
-                       "run_kayobe_playbooks")
     @mock.patch.object(commands.KollaAnsibleMixin,
                        "run_kolla_ansible_overcloud")
-    def test_overcloud_database_recover_force_host(self, mock_kolla_run,
-                                                   mock_run):
+    def test_overcloud_database_recover_force_host(self, mock_run):
         command = commands.OvercloudDatabaseRecover(TestApp(), [])
         parser = command.get_parser("test")
         parsed_args = parser.parse_args(["--force-recovery-host", "foo"])
         result = command.run(parsed_args)
         self.assertEqual(0, result)
-
-        expected_calls = [
-            mock.call(
-                mock.ANY,
-                [utils.get_data_files_path("ansible", "kolla-ansible.yml")],
-                tags="config",
-                ignore_limit=True,
-            ),
-            mock.call(
-                mock.ANY,
-                [
-                    utils.get_data_files_path("ansible",
-                                              "kolla-openstack.yml"),
-                ],
-                ignore_limit=True,
-            ),
-        ]
-        self.assertEqual(expected_calls, mock_run.call_args_list)
-
         expected_calls = [
             mock.call(
                 mock.ANY,
@@ -1348,7 +1310,7 @@ class TestCase(unittest.TestCase):
                 }
             ),
         ]
-        self.assertEqual(expected_calls, mock_kolla_run.call_args_list)
+        self.assertEqual(expected_calls, mock_run.call_args_list)
 
     @mock.patch.object(commands.KayobeAnsibleMixin,
                        "run_kayobe_playbooks")
@@ -1488,6 +1450,44 @@ class TestCase(unittest.TestCase):
         ]
         self.assertEqual(expected_calls, mock_kolla_run.call_args_list)
 
+    @mock.patch.object(commands.KayobeAnsibleMixin,
+                       "run_kayobe_playbooks")
+    @mock.patch.object(commands.KollaAnsibleMixin,
+                       "run_kolla_ansible_overcloud")
+    def test_overcloud_service_prechecks(self, mock_kolla_run, mock_run):
+        command = commands.OvercloudServicePrechecks(TestApp(), [])
+        parser = command.get_parser("test")
+        parsed_args = parser.parse_args([])
+
+        result = command.run(parsed_args)
+        self.assertEqual(0, result)
+
+        expected_calls = [
+            mock.call(
+                mock.ANY,
+                [utils.get_data_files_path("ansible", "kolla-ansible.yml")],
+                ignore_limit=True,
+                tags="config",
+            ),
+            mock.call(
+                mock.ANY,
+                [
+                    utils.get_data_files_path("ansible",
+                                              "kolla-openstack.yml"),
+                ],
+                ignore_limit=True,
+            ),
+        ]
+        self.assertEqual(expected_calls, mock_run.call_args_list)
+
+        expected_calls = [
+            mock.call(
+                mock.ANY,
+                "prechecks",
+            ),
+        ]
+        self.assertEqual(expected_calls, mock_kolla_run.call_args_list)
+
     @mock.patch.object(commands.KayobeAnsibleMixin,
                        "run_kayobe_playbooks")
     @mock.patch.object(commands.KollaAnsibleMixin,
@@ -1733,6 +1733,7 @@ class TestCase(unittest.TestCase):
                     "container_image_sets": (
                         "{{ overcloud_container_image_sets }}"),
                     "push_images": False,
+                    "nocache": False
                 }
             ),
         ]
@@ -1759,6 +1760,7 @@ class TestCase(unittest.TestCase):
                 extra_vars={
                     "container_image_regexes": "^regex1$ ^regex2$",
                     "push_images": True,
+                    "nocache": False
                 }
             ),
         ]
